#!/bin/bash
set -x

handle_error() {
	echo "Error occurred. Exit flashing."
	exit 1
}

trap 'handle_error' ERR

img_dir=$(pwd)

function fastboot_flash_ohp()
{
	fastboot flash fpt "${img_dir}/flash_subimg/fpt.img"

	fastboot flash HSM_FW "${img_dir}/flash_subimg/HSM_FW.img"
	fastboot flash HSM_FW_bak "${img_dir}/flash_subimg/HSM_FW.img"

	fastboot flash HSM_RCA "${img_dir}/flash_subimg/HSM_RCA.img"
	fastboot flash HSM_RCA_bak "${img_dir}/flash_subimg/HSM_RCA.img"

	fastboot flash keyimage "${img_dir}/flash_subimg/keyimage_ohp.img"
	fastboot flash keyimage_bak "${img_dir}/flash_subimg/keyimage_ohp.img"

	fastboot flash SBL "${img_dir}/flash_subimg/SBL.img"
	fastboot flash SBL_bak "${img_dir}/flash_subimg/SBL.img"

	if [ x"${slot_a}" == x"true" ];then
		fastboot flash scp_a "${img_dir}/flash_subimg/scp.img"
	fi
	if [ x"${slot_b}" == x"true" ];then
		fastboot flash scp_b "${img_dir}/flash_subimg/scp.img"
	fi

	if [ x"${slot_a}" == x"true" ];then
		fastboot flash spl_a "${img_dir}/flash_subimg/spl.img"
	fi
	if [ x"${slot_b}" == x"true" ];then
		fastboot flash spl_b "${img_dir}/flash_subimg/spl.img"
	fi

	if [ x"${slot_a}" == x"true" ];then
		fastboot flash MCU_a "${img_dir}/flash_subimg/${mcu_img}"
	fi
	if [ x"${slot_b}" == x"true" ];then
		fastboot flash MCU_b "${img_dir}/flash_subimg/${mcu_img}"
	fi

	fastboot flash misc "${img_dir}/flash_subimg/misc.img"
}

function fastboot_flash_disk()
{
	if [ x"${mode_mtd}" == x"false" ];then
		return
	fi

	echo "this will flash disk img to mtd device"
	#### flash nor ####
	fastboot oem interface:mtd

	if [ x"${security_mode}" == x"true" ];then
		if [ $lifecycle -ge 4 ]; then
			fastboot_flash_ohp
		else
			fastboot.ex flash hb_vspiflash "${img_dir}/disk/${flash_disk_img}" || fastboot oem test:test 2> /dev/null || echo ""
		fi
	else
		fastboot.ex flash hb_vspiflash "${img_dir}/disk/${flash_disk_img}" || fastboot oem test:test 2> /dev/null || echo ""
	fi
}

function fastboot_flash_blk()
{
	if [ x"${mode_blk}" == x"false" ];then
		return
	fi

	echo "this will flash all img to blk device"
	#### flash blk ####
	fastboot oem interface:blk

	if [ x"${mode_disk}" == x"true" ];then
		fastboot flash 0x0 "${img_dir}/disk/${disk_img}" || fastboot oem test:test 2> /dev/null || echo ""
		return 0
	fi

	if [ x"${mode_miniboot}" == x"true" ];then
		fastboot flash 0x0 "${img_dir}/disk/${miniboot_img}" || fastboot oem test:test 2> /dev/null || echo ""
		return 0
	fi
}

mode_disk=" "
mode_mtd=" "
mode_blk=" "
mode_miniboot=" "
slot_a=" "
slot_b=" "

bootintf=""
secflag=""

security_mode_force=""
mcu_img_force=""
lifecycle=

miniboot_img=""
disk_img=""

while getopts ":m:s:f:r:" opt; do
case $opt in
	m)
	flash_mode=$OPTARG
	;;
	s)
	slot_mode=$OPTARG
	;;
	f)
	mcu_img_force=$OPTARG
	;;
	r)
	security_mode_force=$OPTARG
	;;
	\?)
	echo "Invalid option: -$OPTARG" >&2
	;;
esac
done

case $flash_mode in
	"all")
		echo "fastboot flash mode [$flash_mode]"
		mode_blk="true"
		mode_disk="true"
		mode_mtd="true"
		mode_miniboot="false"
		;;
	"mtd")
		echo "fastboot flash mode [$flash_mode]"
		mode_blk="false"
		mode_disk="false"
		mode_mtd="true"
		mode_miniboot="false"
		;;
	"miniboot")
		echo "fastboot flash mode [$flash_mode]"
		mode_blk="true"
		mode_disk="false"
		mode_mtd="false"
		mode_miniboot="true"
		;;
	*)
		echo "fastboot flash mode [default]"
		mode_blk="true"
		mode_disk="true"
		mode_mtd="true"
		mode_miniboot="false"
		;;
esac

case $slot_mode in
	"a")
		echo "fastboot slot mode [$slot_mode]"
		slot_a="true"
		slot_b="false"
		;;
	"b")
		echo "fastboot slot mode [$slot_mode]"
		slot_a="false"
		slot_b="true"
		;;
	*)
		echo "fastboot flash mode [default]"
		slot_a="true"
		slot_b="true"
		;;
esac

function fastboot_get_board_env()
{
	bootintf=$(fastboot getvar bootintf 2>&1 | grep bootintf | cut -d' ' -f2 | tr -d '\r')
	secflag=$(fastboot getvar secflag 2>&1 | grep secflag | cut -d' ' -f2 | tr -d '\r')
	lifecycle=$(fastboot getvar lifecycle 2>&1 | grep lifecycle | cut -d' ' -f2 | tr -d '\r')
	echo "fastboot get board env..."
	echo "bootintf=[$bootintf], secflag=[$secflag], lifecycle=[$lifecycle]"
}


function fastboot_set_mode()
{
	##set security_mode ##
	if [[ -n $security_mode_force ]];then
		security_mode=$security_mode_force
	else
		if [[ -n $secflag ]];then
			if [ 1 -eq $secflag ];then
				security_mode="true"
			elif [ 0 -eq $secflag ];then
				security_mode="false"
			fi
		else
			echo "warning: could not detect board security_mode,use default security_mode[false]"
			security_mode="false"
		fi
	fi

	##set mcu_img##
	if [[ -n $mcu_img_force ]];then
		mcu_img=$mcu_img_force
	else
		mcu_img="MCU_S100_V1.0.img"
	fi

	echo "fastboot set mode..."
	echo "security_mode=[$security_mode], mcu_img=[$mcu_img]"

	##set gpt_img ##
	if [[ -n ${bootintf} ]];then
		if [ x"${bootintf}" == x"scsi" ];then
			miniboot_img="miniboot_ufs.img"
			disk_img="ufs_disk.simg"
		elif [ x"${bootintf}" == x"mmc" ];then
			miniboot_img="miniboot_emmc.img"
			disk_img="emmc_disk.simg"
		fi
	else
		echo "warning: could not detect board bootdevice_mode,use default bootdevice mode[scsi]"
		miniboot_img="miniboot_ufs.img"
		disk_img="ufs_disk.simg"
	fi
	echo "miniboot img=[$miniboot_img]"
	echo "disk img=[$disk_img]"

	##set flash disk img ##
	if [ x"${security_mode}" == x"true" ];then
		flash_disk_img="miniboot_flash.img"
	else
		flash_disk_img="miniboot_flash_nose.img"
	fi
	echo "flash disk img=[$flash_disk_img]"
}

fastboot_get_board_env
fastboot_set_mode
fastboot_flash_disk
fastboot_flash_blk

#### set reboot action ####
fastboot reboot

echo "===================================="
echo "flash all all done"
exit 0
