@echo off
setlocal enabledelayedexpansion

set "bin_path=out\s100"

if "%~1" == "" (
    echo Usage: %~nx0 MCU_IMAGE_NAME
    echo Example: %~nx0 MCU_S100_V1.0.img
    set /p=Hit ENTER to continue...
    exit /b 1
)

set "mcu_bin=%~1%"

set hb_usb_vid=3652

echo waiting for bootrom start...
call :WaitForDFU 6610 10
if errorlevel 1 (
    exit /b 1
)
echo ------------ start download sbl ------------
dfu-util -d,%hb_usb_vid%:6610 -a 0 -D "%bin_path%\cmd_load_sbl"
dfu-util -d,%hb_usb_vid%:6610 -a 0 -D "%bin_path%\sbl.pkg"
dfu-util -d,%hb_usb_vid%:6610 -a 0 -D "%bin_path%\cmd_exit_sbl"
echo waiting for sbl start...
call :WaitForDFU 6620 5
if errorlevel 1 (
    exit /b 1
)


echo ------------ start download spl ------------
dfu-util -d,%hb_usb_vid%:6620 -a 0 -D "%bin_path%\u-boot-spl_ddr.bin" -R
echo waiting for spl start...
call :WaitForDFU 6620 3
if errorlevel 1 (
    exit /b 1
)

echo ------------ start download mcu ------------
dfu-util -d,%hb_usb_vid%:6620 -a 0 -D "%bin_path%\%mcu_bin%" -R
echo waiting for mcu start...
call :WaitForDFU 6625 30
if errorlevel 1 (
    exit /b 1
)

echo ------------ start download bl31 optee uboot ------------
dfu-util -d,%hb_usb_vid%:6625 -a 0 -D "%bin_path%\hobot-s100-bl31.dtb"
dfu-util -d,%hb_usb_vid%:6625 -a 1 -D "%bin_path%\bl31.bin"
dfu-util -d,%hb_usb_vid%:6625 -a 2 -D "%bin_path%\tee-pager_v2.bin"
dfu-util -d,%hb_usb_vid%:6625 -a 3 -D "%bin_path%\u-boot.bin" -R

echo ------------ download all success, start uboot... ------------
set /p=Hit ENTER to continue...

:end
endlocal
exit /b 0

:: Function definition
:WaitForDFU
setlocal
set pid=%1
set timeout=%2
set start_time=%time%

echo Waiting for device %hb_usb_vid%:%pid%, timeout is %timeout% seconds...

:check_loop
:: Get current time and calculate elapsed seconds
for /f "tokens=1-3 delims=:." %%a in ("%start_time%") do (
    set /a "start_h=%%a, start_m=1%%b-100, start_s=1%%c-100"
)
for /f "tokens=1-3 delims=:." %%a in ("%time%") do (
    set /a "current_h=%%a, current_m=1%%b-100, current_s=1%%c-100"
)
set /a "elapsed=(current_h-start_h)*3600 + (current_m-start_m)*60 + (current_s-start_s)"

:: Run dfu-util and process output directly
for /f "delims=" %%i in ('dfu-util.exe -l 2^>^&1') do (
    echo %%i | findstr /c:"%hb_usb_vid%:%pid%" >nul
    if !errorlevel! equ 0 (
        :: echo Device found: %hb_usb_vid%:%pid%
        ping -n 1 127.0.0.1 >nul
        endlocal & exit /b 0
    )
)

:: Wait 1 second before retry
ping -n 1 127.0.0.1 >nul

:: Check for timeout
if %elapsed% geq %timeout% (
    echo ERROR: Timeout reached after %timeout% seconds!
    echo Device with PID %hb_usb_vid%:%pid% not found.
    echo.
    set /p=Hit ENTER to continue...
    endlocal & exit /b 1
)

goto check_loop
:: end of WaitForDFU
