#!/bin/bash
bin_path=""
dfu_path=""
mcu_bin=""

# replace with your image path: /path/to/your/img_packages
default_bin_path="out/s100"

hb_usb_vid="3652"

if [ $# -eq 0 ]; then
    echo "using default path: $default_bin_path"
    bin_path="$default_bin_path"
else
    bin_path="$1"
fi

if [ x$2 != x"" ];then
    dfu_path="-p $2"
fi

if [ -z "$bin_path" ]; then
    echo "Usage: $0 /path/to/your/binpath"
    exit 0
fi

check_dfu_dev() {
    target_pid="$1"
    if [ -n "$2" ];then
        timeout="$2"
    else
        timeout="60"
    fi

    start_time="$SECONDS"
    now="$SECONDS"

    while [ $((now - start_time)) -lt $timeout ];do
        dev=$(dfu-util -l | grep "${hb_usb_vid}:${target_pid}")
        if [ -n "$dev" ];then
            sleep 1
            return
        fi
        now="$SECONDS"
        sleep 1
    done

    echo "ERROR: Wait ${hb_usb_vid}:${target_pid} timeout for $timeout seconds"
    exit 1
}

check_status() {
    if [[ $2 == *"Download done"* ]]; then
        echo "download '$1' success"
    else
        echo "$2"
        exit 1
    fi
}

get_mcu_img() {
    if [[ $1 == *"SIP V2.0"* ]]; then
        mcu_bin="S100_MCU_V1.0.bin"
    else
        echo "!! no mcu interface or mcu interface not support now !!"
        echo "!! run dfu-util -l to check your board type and add it in this script !!"
        exit 1
    fi
}

# bootrom dfu, start sbl
echo "waiting for bootrom start..."
check_dfu_dev 6610 10
echo "------------ start download sbl ------------"
output=$(dfu-util -d,${hb_usb_vid}:6610 -a 0 -D "$bin_path/cmd_load_sbl" ${dfu_path} 2>&1)
check_status "$bin_path/cmd_load_sbl" "$output"
output=$(dfu-util -d,${hb_usb_vid}:6610 -a 0 -D "$bin_path/sbl.pkg" ${dfu_path} 2>&1)
check_status "$bin_path/sbl.pkg" "$output"
output=$(dfu-util -d,${hb_usb_vid}:6610 -a 0 -D "$bin_path/cmd_exit_sbl" ${dfu_path} 2>&1)
check_status "$bin_path/cmd_exit_sbl" "$output"
echo "waiting for sbl start..."
check_dfu_dev 6620 5


# start spl
echo "------------ start download spl ------------"
output=$(dfu-util -d,${hb_usb_vid}:6620 -a 0 -D "$bin_path/u-boot-spl_ddr.bin" ${dfu_path} -R 2>&1)
check_status "$bin_path/u-boot-spl_ddr.bin" "$output"
echo "waiting for spl start..."
check_dfu_dev 6620 3

# start mcu
echo "------------ start download mcu ------------"
output=$(dfu-util ${dfu_path} -l 2>&1)
get_mcu_img "$output"
output=$(dfu-util -d,${hb_usb_vid}:6620 -a 0 -D "$bin_path/$mcu_bin" ${dfu_path} -R 2>&1)
check_status "$bin_path/$mcu_bin" "$output"
echo "waiting for mcu start..."
check_dfu_dev 6625 30

# sbl dfu, start uboot
echo "------------ start download bl31 optee uboot ------------"
output=$(dfu-util -d,${hb_usb_vid}:6625 -a 0 -D "$bin_path/hobot-s100-bl31.dtb" ${dfu_path} 2>&1)
check_status "$bin_path/hobot-s100-bl31.dtb" "$output"
output=$(dfu-util -d,${hb_usb_vid}:6625 -a 1 -D "$bin_path/bl31.bin" ${dfu_path} 2>&1)
check_status "$bin_path/bl31.bin" "$output"
output=$(dfu-util -d,${hb_usb_vid}:6625 -a 2 -D "$bin_path/tee-pager_v2.bin" ${dfu_path} 2>&1)
check_status "$bin_path/tee-pager_v2.bin" "$output"
output=$(dfu-util -d,${hb_usb_vid}:6625 -a 3 -D "$bin_path/u-boot.bin" ${dfu_path} -R 2>&1)
check_status "$bin_path/u-boot.bin" "$output"

echo "------------ download all success, start uboot... ------------"
